module StateMachine
  ( StateMachine(..)
  , stateMachine
  )
 where

import Lava

----------------------------------------------------------------
-- StateMachine datatype

data StateMachine state inp 
  = StateMachine
      { states     :: [state]
      , inputs     :: [inp]
      , initial    :: [state]
      , transition :: state -> inp -> [state]
      }

-- for the translation

type StateCircuit
  = [Signal Bool] -> [Signal Bool]

----------------------------------------------------------------
-- StateMachine translation

stateMachine :: (Eq inp, Eq state) => StateMachine state inp -> StateCircuit
stateMachine machine inSignals = outSignals
  where
    component state = (active, emits)
      where
        init   = state `elem` initial machine
        active = delay (bool init) (activating state)
        
        emits  =
          [ ( state'
            , and2 (active, inSignal input)
            )
          | input  <- inputs machine
          , state' <- transition machine state input
          ]

    components =
      [ component state
      | state <- states machine
      ]
    
    activating state =
      orl [ activate
          | (_, emits)         <- components
          , (state', activate) <- emits
          , state == state'
          ]
    
    inSignal input =
      head [ sig
           | (input',sig) <- inputs machine `zip` inSignals
           , input == input'
           ]

    outSignals =
      [ active
      | (active, _) <- components
      ]

----------------------------------------------------------------
-- the end.

